import java.util.ArrayList;
import java.util.List;

import junit.framework.AssertionFailedError;

import org.junit.Assert;
import org.junit.Ignore;
import org.junit.internal.AssumptionViolatedException;
import org.junit.internal.runners.model.EachTestNotifier;
import org.junit.runner.Description;
import org.junit.runner.Result;
import org.junit.runner.Runner;
import org.junit.runner.notification.Failure;
import org.junit.runner.notification.RunNotifier;
import org.junit.runners.BlockJUnit4ClassRunner;
import org.junit.runners.model.FrameworkMethod;
import org.junit.runners.model.InitializationError;
import org.junit.runners.model.Statement;


public class MultiAssertRunner extends Runner {
	
	public static class AssertionResult {

		private final String message;
		private final boolean passed;

		public AssertionResult(String message, boolean passed) {
			this.message = message;
			this.passed = passed;
		}

	}

	private static List<AssertionResult> assertions = new ArrayList<AssertionResult>();
	private BlockJUnit4ClassRunner innerRunner;

	public MultiAssertRunner(Class<?> testClass) throws InitializationError {
		innerRunner = new BlockJUnit4ClassRunner(testClass) {
			@Override
			protected void runChild(FrameworkMethod method, RunNotifier notifier) {
				Description description= describeChild(method);
				if (method.getAnnotation(Ignore.class) != null) {
					notifier.fireTestIgnored(description);
				} else {
					ourRunLeaf(method, notifier);
				}
			}
			protected void ourRunLeaf(FrameworkMethod method, RunNotifier notifier) {
				try {
					methodBlock(method).evaluate();
				} catch (Throwable e) {
					Description description= describeChild(method);
					EachTestNotifier eachNotifier= new EachTestNotifier(notifier, description);
					eachNotifier.fireTestStarted();
					eachNotifier.addFailure(e);
					eachNotifier.fireTestFinished();
					return;
				}
				
				Description methodSuiteDescription= Description.createSuiteDescription(
						testName(method) + "(" + getTestClass().getJavaClass().getName() + ")",
						method.getAnnotations());
				notifier.fireTestStarted(methodSuiteDescription);
				for (AssertionResult assertionResult : assertions) {
					Description description = Description.createSuiteDescription(
							testName(method) + "[" + assertionResult.message + "]" +
									"(" + getTestClass().getJavaClass().getName() + ")",
							method.getAnnotations());
					notifier.fireTestStarted(description);
					if (!assertionResult.passed) {
						notifier.fireTestFailure(new Failure(description, new AssertionFailedError(assertionResult.message)));
					}
					notifier.fireTestFinished(description);
				}
				notifier.fireTestFinished(methodSuiteDescription);
			}
			
			@Override
			protected String getName() {
				return super.getName();
			}
		};
	}


	@Override
	public Description getDescription() {
		return innerRunner.getDescription();
	}

	@Override
	public void run(RunNotifier notifier) {
		innerRunner.run(notifier);
	}

	public static void assertEquals(String expected, String actual) {
		String assertionName = "Not equals <" + expected + "> but was <" + actual + ">";
		boolean passed = (expected == actual) || (expected != null && expected.equals(actual));
		addAssertionResult(assertionName, passed);
	}

	private static boolean addAssertionResult(String message, boolean passed) {
		return assertions.add(new AssertionResult(message, passed));
	}

}
